/*
 *  This file is part of uIRC. (https://git.redxen.eu/caskd/uIRC)
 *  Copyright (c) 2019-2021 Alex-David Denes
 *
 *  uIRC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  uIRC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with uIRC.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "assembler.h" // uirc_assembler_*
#include "error.h"     // uirc_errno
#include "type.h"      // IRC_Message IRC_Tag IRC_User

#include <assert.h>    // assert()
#include <stdio.h>     // NULL, snprintf()
#include <sys/types.h> // size_t

ssize_t
uirc_assembler_message(char* buf, const IRC_Message* m, size_t len)
{
	assert(buf != NULL);
	assert(m != NULL);

	const char* const sv = buf;

#ifdef UIRC_FEATURE_IRCV3
	if (m->tag_list != NULL) {
		if (len > 1) {
			*(buf++) = '@';
			len--;
		} else {
			uirc_errno = UIRC_ERR_BUFSIZ;
			return -1;
		}

		ssize_t ret;
		if ((ret = uirc_assembler_tag_list(buf, m->tag_list, len)) >= 0) {
			buf += (size_t) ret;
			len -= (size_t) ret;
		} else
			return ret;

		if (len > 1) {
			*(buf++) = ' ';
			len--;
		} else {
			uirc_errno = UIRC_ERR_BUFSIZ;
			return -1;
		}
	}
	assert(sv <= buf);
#endif /* UIRC_FEATURE_IRCV3 */
	if (m->source != NULL && (m->source->nick != NULL || m->source->host != NULL)) {
		if (len > 1) {
			*(buf++) = ':';
			len--;
		} else {
			uirc_errno = UIRC_ERR_BUFSIZ;
			return -1;
		}

		ssize_t ret;
		if ((ret = uirc_assembler_user(buf, m->source, len)) >= 0) {
			buf += (size_t) ret;
			len -= (size_t) ret;
		} else
			return ret;

		if (len > 1) {
			*(buf++) = ' ';
			len--;
		} else {
			uirc_errno = UIRC_ERR_BUFSIZ;
			return -1;
		}
	}
	assert(sv <= buf);

	if (m->command != NULL) {
		int ret;
		if ((ret = snprintf(buf, len, "%s", m->command)) >= 0) {
			buf += (size_t) ret;
			len -= (size_t) ret;
		} else {
			uirc_errno = UIRC_ERR_BUFSIZ;
			return -1;
		}
	} else {
		uirc_errno = UIRC_ERR_INVFMT;
		return -1;
	}
	assert(sv <= buf);

	for (unsigned int i = 0; i < IRC_MAXARGS && m->args[i] != NULL; i++) {
		int ret;
		if ((ret = snprintf(buf, len, (m->args[i + 1] == NULL && m->trailing) ? " :%s" : " %s", m->args[i])) >= 0) {
			buf += (size_t) ret;
			len -= (size_t) ret;
		} else {
			uirc_errno = UIRC_ERR_BUFSIZ;
			return -1;
		}
	}
	assert(sv <= buf);

	int ret;
	if ((ret = snprintf(buf, len, "\r\n")) >= 0) {
		buf += (size_t) ret;
		len -= (size_t) ret;
	} else {
		uirc_errno = UIRC_ERR_BUFSIZ;
		return -1;
	}
	assert(sv <= buf);

	return (ssize_t)(buf - sv);
}

